/***********************************************************************
Report standard features of C (K&R, C89, C99) and C++ (traditional or
1998) implementations: predefined macros, header files, data types,
and sizes of data types.
[21-Mar-2001]
***********************************************************************/

#include "config.h"
#include "headers.h"

#define PRINTF (void)printf

int
main(int argc, char* argv[])
{

#if defined(CC)
    PRINTF("Compiler                             = %s\n", CC);
    PRINTF("\n\n");
#endif

    PRINTF("Available standard predefined constant macros...\n");

#if defined(__bool_true_false_are_defined)
    PRINTF("\t__bool_true_false_are_defined = %8ld\n", (long)(__bool_true_false_are_defined + 0L));
#endif

#if defined(__cplusplus)
    PRINTF("\t__cplusplus                  = %8ld\n", (long) (__cplusplus + 0L));
#endif

#if defined(__DATE__)
    PRINTF("\t__DATE__                     = \"%s\"\n", __DATE__);
#endif

#if defined(__FILE__)
    PRINTF("\t__FILE__                     = \"%s\"\n", __FILE__);
#endif

#if defined(__func__)
    PRINTF("\t__func__                     = \"%s\"\n", __func__);
#endif

#if defined(__LINE__)
    PRINTF("\t__LINE__                     = %8ld\n", (long) (__LINE__ + 0L));
#endif

#if defined(__STDC__)
    PRINTF("\t__STDC__                     = %8ld\n", (long) (__STDC__ + 0L));
#endif

#if defined(__STDC_HOSTED__)
    PRINTF("\t__STDC_HOSTED__              = %8ld\n", (long) (__STDC_HOSTED__ + 0L));
#endif

#if defined(__STDC_IEC_559__)
    PRINTF("\t__STDC_IEC_559__             = %8ld\n", (long) (__STDC_IEC_559__ + 0L));
#endif

#if defined(__STDC_IEC_559_COMPLEX__)
    PRINTF("\t__STDC_IEC_559_COMPLEX__     = %8ld\n", (long) (__STDC_IEC_559_COMPLEX__ + 0L));
#endif

#if defined(__STDC_ISO_10646__)
    PRINTF("\t__STDC_ISO_10646__           = %8ld\n", (long) (__STDC_ISO_10646__ + 0L));
#endif

#if defined(__STDC_CONSTANT_MACROS)
    PRINTF("\t__STDC_CONSTANT_MACROS       = %8ld\n", (long) (__STDC_CONSTANT_MACROS + 0L));
#endif

#if defined(__STDC_FORMAT_MACROS)
    PRINTF("\t__STDC_FORMAT_MACROS         = %8ld\n", (long) (__STDC_FORMAT_MACROS + 0L));
#endif

#if defined(__STDC_LIMIT_MACROS)
    PRINTF("\t__STDC_LIMIT_MACROS          = %8ld\n", (long) (__STDC_LIMIT_MACROS + 0L));
#endif

#if defined(__STDC_VERSION__)
    PRINTF("\t__STDC_VERSION__             = %8ld\n", (long) (__STDC_VERSION__ + 0L));
#endif

#if defined(__TIME__)
    PRINTF("\t__TIME__                     = \"%s\"\n", __TIME__);
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Missing standard predefined constant macros...\n");

#if !defined(__bool_true_false_are_defined)
    PRINTF("\t__bool_true_false_are_defined\n");
#endif

#if !defined(__cplusplus)
    PRINTF("\t__cplusplus\n");
#endif

#if !defined(__DATE__)
    PRINTF("\t__DATE__\n");
#endif

#if !defined(__FILE__)
    PRINTF("\t__FILE__\n");
#endif

#if !defined(__func__)
    PRINTF("\t__func__\n");
#endif

#if !defined(__LINE__)
    PRINTF("\t__LINE__\n");
#endif

#if !defined(__STDC__)
    PRINTF("\t__STDC__\n");
#endif

#if !defined(__STDC_HOSTED__)
    PRINTF("\t__STDC_HOSTED__\n");
#endif

#if !defined(__STDC_IEC_559__)
    PRINTF("\t__STDC_IEC_559__\n");
#endif

#if !defined(__STDC_IEC_559_COMPLEX__)
    PRINTF("\t__STDC_IEC_559_COMPLEX__\n");
#endif

#if !defined(__STDC_ISO_10646__)
    PRINTF("\t__STDC_ISO_10646__\n");
#endif

#if !defined(__STDC_CONSTANT_MACROS)
    PRINTF("\t__STDC_CONSTANT_MACROS\n");
#endif

#if !defined(__STDC_FORMAT_MACROS)
    PRINTF("\t__STDC_FORMAT_MACROS\n");
#endif

#if !defined(__STDC_LIMIT_MACROS)
    PRINTF("\t__STDC_LIMIT_MACROS\n");
#endif

#if !defined(__STDC_VERSION__)
    PRINTF("\t__STDC_VERSION__\n");
#endif

#if !defined(__TIME__)
    PRINTF("\t__TIME__\n");
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Available standard header files...\n");

#if defined(HAVE_FENV_H)
    PRINTF("\t<fenv.h>\n");
#endif

#if defined(HAVE_INTTYPES_H)
    PRINTF("\t<inttypes.h>\n");
#endif

#if defined(HAVE_MATH_H)
    PRINTF("\t<math.h>\n");
#endif

#if defined(HAVE_SIGNAL_H)
    PRINTF("\t<signal.h>\n");
#endif

#if defined(HAVE_STDBOOL_H)
    PRINTF("\t<stdbool.h>\n");
#endif

#if defined(HAVE_STDDEF_H)
    PRINTF("\t<stddef.h>\n");
#endif

#if defined(HAVE_STDINT_H)
    PRINTF("\t<stdint.h>\n");
#endif

#if defined(HAVE_STDIO_H)
    PRINTF("\t<stdio.h>\n");
#endif

#if defined(HAVE_STDLIB_H)
    PRINTF("\t<stdlib.h>\n");
#endif

#if defined(HAVE_SYS_TYPES_H)
    PRINTF("\t<sys/types.h>\n");
#endif

#if defined(HAVE_TIME_H)
    PRINTF("\t<time.h>\n");
#endif

#if defined(HAVE_UNISTD_H)
    PRINTF("\t<unistd.h>\n");
#endif

#if defined(HAVE_WCHAR_H)
    PRINTF("\t<wchar.h>\n");
#endif

#if defined(HAVE_WCTYPE_H)
    PRINTF("\t<wctype.h>\n");
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Missing standard header files...\n");

#if !defined(HAVE_FENV_H)
    PRINTF("\t<fenv.h>\n");
#endif

#if !defined(HAVE_INTTYPES_H)
    PRINTF("\t<inttypes.h>\n");
#endif

#if !defined(HAVE_MATH_H)
    PRINTF("\t<math.h>\n");
#endif

#if !defined(HAVE_SIGNAL_H)
    PRINTF("\t<signal.h>\n");
#endif

#if !defined(HAVE_STDBOOL_H)
    PRINTF("\t<stdbool.h>\n");
#endif

#if !defined(HAVE_STDDEF_H)
    PRINTF("\t<stddef.h>\n");
#endif

#if !defined(HAVE_STDINT_H)
    PRINTF("\t<stdint.h>\n");
#endif

#if !defined(HAVE_STDIO_H)
    PRINTF("\t<stdio.h>\n");
#endif

#if !defined(HAVE_STDLIB_H)
    PRINTF("\t<stdlib.h>\n");
#endif

#if !defined(HAVE_SYS_TYPES_H)
    PRINTF("\t<sys/types.h>\n");
#endif

#if !defined(HAVE_TIME_H)
    PRINTF("\t<time.h>\n");
#endif

#if !defined(HAVE_UNISTD_H)
    PRINTF("\t<unistd.h>\n");
#endif

#if !defined(HAVE_WCHAR_H)
    PRINTF("\t<wchar.h>\n");
#endif

#if !defined(HAVE_WCTYPE_H)
    PRINTF("\t<wctype.h>\n");
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Pointer sizes...\n");
    PRINTF("\tsizeof(char*)                \t= %4u\n", sizeof(char*));
    PRINTF("\tsizeof(void*)                \t= %4u\n", sizeof(void*));
    PRINTF("\tsizeof(void(*)(void))        \t= %4u\n", sizeof(void(*)(void)));
    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Integer sizes...\n");
    PRINTF("\tsizeof(char)                 \t= %4u\n", sizeof(char));
    PRINTF("\tsizeof(short)                \t= %4u\n", sizeof(short));
    PRINTF("\tsizeof(int)                  \t= %4u\n", sizeof(int));
    PRINTF("\tsizeof(long)                 \t= %4u\n", sizeof(long));

#if defined(HAVE_LONG_LONG)
    PRINTF("\tsizeof(long long)            \t= %4u\n", sizeof(long long));
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Integer-like sizes...\n");

#if defined(HAVE_CLOCK_T)
    PRINTF("\tsizeof(clock_t)              \t= %4u\n", sizeof(clock_t));
#endif

#if defined(HAVE_DIV_T)
    PRINTF("\tsizeof(div_t)                \t= %4u\n", sizeof(div_t));
#endif

#if defined(HAVE_FPOS_T)
    PRINTF("\tsizeof(fpos_t)               \t= %4u\n", sizeof(fpos_t));
#endif

#if defined(HAVE_INTMAX_T)
    PRINTF("\tsizeof(intmax_t)             \t= %4u\n", sizeof(intmax_t));
#endif

#if defined(HAVE_LDIV_T)
    PRINTF("\tsizeof(ldiv_t)               \t= %4u\n", sizeof(ldiv_t));
#endif

#if defined(HAVE_OFF_T)
    PRINTF("\tsizeof(off_t)                \t= %4u\n", sizeof(off_t));
#endif

#if defined(HAVE_PTRDIFF_T)
    PRINTF("\tsizeof(ptrdiff_t)            \t= %4u\n", sizeof(ptrdiff_t));
#endif

#if defined(HAVE_SIG_ATOMIC_T)
    PRINTF("\tsizeof(sig_atomic_t)         \t= %4u\n", sizeof(sig_atomic_t));
#endif

#if defined(HAVE_SIZE_T)
    PRINTF("\tsizeof(size_t)               \t= %4u\n", sizeof(size_t));
#endif

#if defined(HAVE_TIME_T)
    PRINTF("\tsizeof(time_t)               \t= %4u\n", sizeof(time_t));
#endif

#if defined(HAVE_UINTMAX_T)
    PRINTF("\tsizeof(uintmax_t)            \t= %4u\n", sizeof(uintmax_t));
#endif

#if defined(HAVE_WCHAR_T)
    PRINTF("\tsizeof(wchar_t)              \t= %4u\n", sizeof(wchar_t));
#endif

    PRINTF("\n\n");

    /* ============================================================== */
    PRINTF("C++ 1998 integer-like sizes...\n");

#if defined(HAVE_NOTHROW_T)
    PRINTF("\tsizeof(nothrow_t)            \t= %4u\n", sizeof(nothrow_t));
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("C99 integer-like sizes...\n");

#if defined(HAVE_DOUBLE_T)
    PRINTF("\tsizeof(double_t)             \t= %4u\n", sizeof(double_t));
#endif

#if defined(HAVE_FENV_T)
    PRINTF("\tsizeof(fenv_t)               \t= %4u\n", sizeof(fenv_t));
#endif

#if defined(HAVE_FEXCEPT_T)
    PRINTF("\tsizeof(fexcept_t)            \t= %4u\n", sizeof(fexcept_t));
#endif

#if defined(HAVE_FLOAT_T)
    PRINTF("\tsizeof(float_t)              \t= %4u\n", sizeof(float_t));
#endif

#if defined(HAVE_IMAXDIV_T)
    PRINTF("\tsizeof(imaxdiv_t)            \t= %4u\n", sizeof(imaxdiv_t));
#endif

#if defined(HAVE_INT8_T)
    PRINTF("\tsizeof(int8_t)               \t= %4u\n", sizeof(int8_t));
#endif

#if defined(HAVE_INT_FAST16_T)
    PRINTF("\tsizeof(int_fast16_t)         \t= %4u\n", sizeof(int_fast16_t));
#endif

#if defined(HAVE_INT_FAST32_T)
    PRINTF("\tsizeof(int_fast32_t)         \t= %4u\n", sizeof(int_fast32_t));
#endif

#if defined(HAVE_INT_FAST64_T)
    PRINTF("\tsizeof(int_fast64_t)         \t= %4u\n", sizeof(int_fast64_t));
#endif

#if defined(HAVE_INT_FAST8_T)
    PRINTF("\tsizeof(int_fast8_t)          \t= %4u\n", sizeof(int_fast8_t));
#endif

#if defined(HAVE_INT_LEAST16_T)
    PRINTF("\tsizeof(int_least16_t)        \t= %4u\n", sizeof(int_least16_t));
#endif

#if defined(HAVE_INT_LEAST32_T)
    PRINTF("\tsizeof(int_least32_t)        \t= %4u\n", sizeof(int_least32_t));
#endif

#if defined(HAVE_INT_LEAST64_T)
    PRINTF("\tsizeof(int_least64_t)        \t= %4u\n", sizeof(int_least64_t));
#endif

#if defined(HAVE_INT_LEAST8_T)
    PRINTF("\tsizeof(int_least8_t)         \t= %4u\n", sizeof(int_least8_t));
#endif

#if defined(HAVE_INTPTR_T)
    PRINTF("\tsizeof(intptr_t)             \t= %4u\n", sizeof(intptr_t));
#endif

#if defined(HAVE_LLDIV_T)
    PRINTF("\tsizeof(lldiv_t)              \t= %4u\n", sizeof(lldiv_t));
#endif

#if defined(HAVE_MBSTATE_T)
    PRINTF("\tsizeof(mbstate_t)            \t= %4u\n", sizeof(mbstate_t));
#endif

#if defined(HAVE_SSIZE_T)
    PRINTF("\tsizeof(ssize_t)              \t= %4u\n", sizeof(ssize_t)); /* Posix.2 */
#endif

#if defined(HAVE_UINT_FAST16_T)
    PRINTF("\tsizeof(uint_fast16_t)        \t= %4u\n", sizeof(uint_fast16_t));
#endif

#if defined(HAVE_UINT_FAST32_T)
    PRINTF("\tsizeof(uint_fast32_t)        \t= %4u\n", sizeof(uint_fast32_t));
#endif

#if defined(HAVE_UINT_FAST64_T)
    PRINTF("\tsizeof(uint_fast64_t)        \t= %4u\n", sizeof(uint_fast64_t));
#endif

#if defined(HAVE_UINT_FAST8_T)
    PRINTF("\tsizeof(uint_fast8_t)         \t= %4u\n", sizeof(uint_fast8_t));
#endif

#if defined(HAVE_UINT_LEAST16_T)
    PRINTF("\tsizeof(uint_least16_t)       \t= %4u\n", sizeof(uint_least16_t));
#endif

#if defined(HAVE_UINT_LEAST32_T)
    PRINTF("\tsizeof(uint_least32_t)       \t= %4u\n", sizeof(uint_least32_t));
#endif

#if defined(HAVE_UINT_LEAST64_T)
    PRINTF("\tsizeof(uint_least64_t)       \t= %4u\n", sizeof(uint_least64_t));
#endif

#if defined(HAVE_UINT_LEAST8_T)
    PRINTF("\tsizeof(uint_least8_t)        \t= %4u\n", sizeof(uint_least8_t));
#endif

#if defined(HAVE_UINTPTR_T)
    PRINTF("\tsizeof(uintptr_t)            \t= %4u\n", sizeof(uintptr_t));
#endif

#if defined(HAVE_WCHAR_T)
    PRINTF("\tsizeof(wchar_t)              \t= %4u\n", sizeof(wchar_t));
#endif

#if defined(HAVE_WCTRANS_T)
    PRINTF("\tsizeof(wctrans_t)            \t= %4u\n", sizeof(wctrans_t));
#endif

#if defined(HAVE_WCTYPE_T)
    PRINTF("\tsizeof(wctype_t)             \t= %4u\n", sizeof(wctype_t));
#endif

#if defined(HAVE_WINT_T)
    PRINTF("\tsizeof(wint_t)               \t= %4u\n", sizeof(wint_t));
#endif

#if defined(HAVE__BOOL)
    PRINTF("\tsizeof(_Bool)                \t= %4u\n", sizeof(_Bool));
#endif

#if defined(HAVE__COMPLEX)
    PRINTF("\tsizeof(_Complex)             \t= %4u\n", sizeof(_Complex));
#endif

#if defined(HAVE__IMAGINARY)
    PRINTF("\tsizeof(_Imaginary)           \t= %4u\n", sizeof(_Imaginary));
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Floating-point sizes...\n");
    PRINTF("\tsizeof(float)                \t= %4u\n", sizeof(float));
    PRINTF("\tsizeof(double)               \t= %4u\n", sizeof(double));

#if defined(HAVE_LONG_DOUBLE)
    PRINTF("\tsizeof(long double)          \t= %4u\n", sizeof(long double));
#endif

    PRINTF("\n\n");

    /* ============================================================== */

    PRINTF("Missing data types...\n");

#if !defined(HAVE_CLOCK_T)
    PRINTF("\tclock_t\n");
#endif

#if !defined(HAVE_DIV_T)
    PRINTF("\tdiv_t\n");
#endif

#if !defined(HAVE_DOUBLE_T)
    PRINTF("\tdouble_t\n");
#endif

#if !defined(HAVE_FENV_T)
    PRINTF("\tfenv_t\n");
#endif

#if !defined(HAVE_FEXCEPT_T)
    PRINTF("\tfexcept_t\n");
#endif

#if !defined(HAVE_FLOAT_T)
    PRINTF("\tfloat_t\n");
#endif

#if !defined(HAVE_FPOS_T)
    PRINTF("\tfpos_t\n");
#endif

#if !defined(HAVE_IMAXDIV_T)
    PRINTF("\timaxdiv_t\n");
#endif

#if !defined(HAVE_INT8_T)
    PRINTF("\tint8_t\n");
#endif

#if !defined(HAVE_INTMAX_T)
    PRINTF("\tintmax_t\n");
#endif

#if !defined(HAVE_INTPTR_T)
    PRINTF("\tintptr_t\n");
#endif

#if !defined(HAVE_INT_FAST16_T)
    PRINTF("\tint_fast16_t\n");
#endif

#if !defined(HAVE_INT_FAST32_T)
    PRINTF("\tint_fast32_t\n");
#endif

#if !defined(HAVE_INT_FAST64_T)
    PRINTF("\tint_fast64_t\n");
#endif

#if !defined(HAVE_INT_FAST8_T)
    PRINTF("\tint_fast8_t\n");
#endif

#if !defined(HAVE_INT_LEAST16_T)
    PRINTF("\tint_least16_t\n");
#endif

#if !defined(HAVE_INT_LEAST32_T)
    PRINTF("\tint_least32_t\n");
#endif

#if !defined(HAVE_INT_LEAST64_T)
    PRINTF("\tint_least64_t\n");
#endif

#if !defined(HAVE_INT_LEAST8_T)
    PRINTF("\tint_least8_t\n");
#endif

#if !defined(HAVE_LDIV_T)
    PRINTF("\tldiv_t\n");
#endif

#if !defined(HAVE_LLDIV_T)
    PRINTF("\tlldiv_t\n");
#endif

#if !defined(HAVE_LONG_DOUBLE)
    PRINTF("\tlong double\n");
#endif

#if !defined(HAVE_LONG_LONG)
    PRINTF("\tlong long\n");
#endif

#if !defined(HAVE_MBSTATE_T)
    PRINTF("\tmbstate_t\n");
#endif

#if !defined(HAVE_NOTHROW_T)
    PRINTF("\tnothrow_t\n");
#endif

#if !defined(HAVE_OFF_T)
    PRINTF("\toff_t\n");
#endif

#if !defined(HAVE_PTRDIFF_T)
    PRINTF("\tptrdiff_t\n");
#endif

#if !defined(HAVE_SIG_ATOMIC_T)
    PRINTF("\tsig_atomic_t\n");
#endif

#if !defined(HAVE_SIZE_T)
    PRINTF("\tsize_t\n");
#endif

#if !defined(HAVE_SSIZE_T)
    PRINTF("\tssize_t\n");
#endif

#if !defined(HAVE_TIME_T)
    PRINTF("\ttime_t\n");
#endif

#if !defined(HAVE_UINTMAX_T)
    PRINTF("\tuintmax_t\n");
#endif

#if !defined(HAVE_UINTPTR_T)
    PRINTF("\tuintptr_t\n");
#endif

#if !defined(HAVE_UINT_FAST16_T)
    PRINTF("\tuint_fast16_t\n");
#endif

#if !defined(HAVE_UINT_FAST32_T)
    PRINTF("\tuint_fast32_t\n");
#endif

#if !defined(HAVE_UINT_FAST64_T)
    PRINTF("\tuint_fast64_t\n");
#endif

#if !defined(HAVE_UINT_FAST8_T)
    PRINTF("\tuint_fast8_t\n");
#endif

#if !defined(HAVE_UINT_LEAST16_T)
    PRINTF("\tuint_least16_t\n");
#endif

#if !defined(HAVE_UINT_LEAST32_T)
    PRINTF("\tuint_least32_t\n");
#endif

#if !defined(HAVE_UINT_LEAST64_T)
    PRINTF("\tuint_least64_t\n");
#endif

#if !defined(HAVE_UINT_LEAST8_T)
    PRINTF("\tuint_least8_t\n");
#endif

#if !defined(HAVE_WCHAR_T)
    PRINTF("\twchar_t\n");
#endif

#if !defined(HAVE_WCHAR_T)
    PRINTF("\twchar_t\n");
#endif

#if !defined(HAVE_WCTRANS_T)
    PRINTF("\twctrans_t\n");
#endif

#if !defined(HAVE_WCTYPE_T)
    PRINTF("\twctype_t\n");
#endif

#if !defined(HAVE_WINT_T)
    PRINTF("\twint_t\n");
#endif

#if !defined(HAVE__BOOL)
    PRINTF("\t_Bool\n");
#endif

#if !defined(HAVE__COMPLEX)
    PRINTF("\t_Complex\n");
#endif

#if !defined(HAVE__IMAGINARY)
    PRINTF("\t_Imaginary\n");
#endif

    PRINTF("\n\n");

    return (EXIT_SUCCESS);
}
